
ccc   Subroutines on "nc_month_routines.f":
ccc    write_month      - computes a 2D monthly mean field, writes this
ccc                       to a netCDF file, and re-initializes sums etc.
ccc    read_set         - reads one 2D field, converts to real*4
ccc                       (if necessary), and updates sums etc.
ccc    copy_lonlat      - copies 2D or 1D longitudes and latitudes from
ccc                       one netCDF file to another.
ccc    get_vartype      - reads and checks input variable type, reads
ccc                       offset and scaling factor, writes offset and
ccc                       scaling factor to output field
ccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc



c     ********************
c     General description:
c     ********************

c     The subroutine "write_month" computes one (unweighed) 2D mean
c     field, and writes the result to the output netCDF file. If this
c     is the first field that is written for a particular month, the
c     time variable for the month no. is also written to the netCDF
c     file. The variables that hold sums and no. of entries are
c     reinitialized.


c     ***********************
c     Input/output variables:
c     ***********************

c      nx, ny         (i)  - dimension sizes of 2D arrays
c      z              (i)  - present level no. (>=1), if value is 1,
c                             write month no. on the netCDF file
c      n_data         (iA) - grid-by-grid no. of values accumulated
c                             in 'sum'
c      sum            (r8A)- accumulated sums of values for the
c                             present month
c      r4_field       (rA) - work array
c      i2_field       (i2A)- work array
c      i4_field       (iA) - work array
c      new_date       (iA) - the origin date on the output netCDF file
c                             w/ monthly mean fields
c      date           (iA) - the date for the next month, which is not
c                             in the present month (triggering
c                             "write_month" to be called)
c      var_type_out   (i)  - the variable type used for the output
c                             field (NF_SHORT, NF_INT, NF_FLOAT)
c      offset_out     (r)  - offset value used in conversion from
c                             floating point to integer numbers
c      scale_out      (r)  - scaling factor used in conversion from
c                             floating point to integer numbers
c      yearno         (i)  - the mean field's year that will be written
c                             here
c      monthno        (i)  - the mean field's month that will be written
c                             here
c      timefactor     (r)  - a scaling factor which depends on the unit
c                             of the month no.s on the output netCDF
c                             file
c      month_id       (i)  - the file ID for the output netCDF file
c      time_id_m      (i)  - the variable ID for the month no.
c      outno          (i)  - the no. of months that have previously
c                             been written to the netCDF file (for the
c                             present level no.), updated here

c     Above:
c      i  - integer variable [ 4 byte, aka. "long" ]
c      iA - integer array    [ 4 byte, aka. "long" ]
c      i2A- integer*2 array  [ 2 byte, aka. "short" ]
c      r  - real variable    [ 4 byte, aka. "float" ]
c      r8A- real*8 array     [ 8 byte, aka. "double" ]


      subroutine write_month(nx,ny,z,n_data,sum,r4_field,i2_field,
     >                       i4_field,new_date,date,var_type_out,
     >                       var_out_id,offset_out,scale_out,
     >                       yearno,monthno,timefactor,
     >                       month_id,time_id_m,outno)

      implicit none

      include 'netcdf.inc'
      include 'missing_values.inc'

c     Input:
      integer   nx,ny
      integer   z,var_type_out,var_out_id,month_id,time_id_m,
     >          new_date(5),date(5)
      real      offset_out,scale_out,timefactor

c     I/O:
      integer   n_data(nx,ny)
      real*8    sum(nx,ny)
      integer   yearno,monthno,outno

c     Work:
      integer*2 i2_field(nx,ny)
      integer   i4_field(nx,ny)
      real      r4_field(nx,ny)

c     Local:
      integer x,y,nc_status,size,start(3),count(3)
      real timevalue

c     Function:
      integer str_len


c     Compute mean:
      do y = 1,ny
         do x = 1,nx
            if (n_data(x,y) .gt. 0) then
               r4_field(x,y) = sngl(sum(x,y)/dble(n_data(x,y)))
            else
               r4_field(x,y) = miss_val_r(1)
            end if
         end do
      end do

c     Update 'outno':
      outno = outno + 1
      start(1) = 1
      start(2) = 1
      start(3) = outno
      count(1) = nx
      count(2) = ny
      count(3) = 1

c     Store month no.?
      if (z .eq. 1) then
         timevalue = 12.*float(yearno  - new_date(1)) +
     >                   float(monthno - new_date(2))
         timevalue = timefactor*timevalue
         nc_status = NF_PUT_VAR1_REAL(month_id, time_id_m, outno,
     >                                timevalue)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status,321)
      end if

c     Convert from floating point numbers?,
c     store mean field as original type:
      if (var_type_out .eq. NF_SHORT) then
         call float2short_2D(nx,ny,r4_field,offset_out,scale_out,
     >                       i2_field)
         nc_status = NF_PUT_VARA_INT2(month_id, var_out_id,
     >                                start, count, i2_field)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status,322)
      else if (var_type_out .eq. NF_INT) then
         call float2int_2D(nx,ny,r4_field,offset_out,scale_out,i4_field)
         nc_status = NF_PUT_VARA_INT(month_id, var_out_id,
     >                               start, count, i4_field)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status,323)
      else
         nc_status = NF_PUT_VARA_REAL(month_id, var_out_id,
     >                                start, count, r4_field)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status,324)
      end if

c     Re-initialize 'yearno', 'monthno', 'n_data', 'sum':
      yearno  = date(1)
      monthno = date(2)
      do y = 1,ny
         do x = 1,nx
            n_data(x,y) = 0
            sum(x,y)    = dble(0.)
         end do
      end do


      return
      end



c     ********************
c     General description:
c     ********************

c     The subroutine "read_set" reads one 2D field, converts to floating
c     point numbers (real*4, if necessary), and updates 'n_data'
c     (grid-by-grid no. of "wet" values) and 'sum' (accumulated sum of
c     "wet" values.


c     ***********************
c     Input/output variables:
c     ***********************

c      nx, ny         (i)  - dimension sizes of 2D arrays
c      i2_field       (i2A)- work array
c      i4_field       (iA) - work array
c      r4_field       (rA) - work array
c      r8_field       (r8A)- work array
c      n_data         (iA) - grid-by-grid no. of values accumulated
c                             in 'sum'
c      sum            (r8A)- accumulated sums of values for the
c                             present month
c      nc_id          (i)  - the file ID for the input netCDF file
c      var_id         (i)  - the variable ID for the input field
c      var_type       (i)  - the input field's variable type
c                             (NF_SHORT, NF_INT, NF_FLOATm NF_DOUBLE)
c      start          (i)  - defines the starting point for reading
c                             results stored on the input file
c      count          (i)  - sets the span when reading one 2D field
c                             from the input file
c      offset         (r)  - offset value used in conversion from
c                             integer input values to real numbers
c      scale          (r)  - scaling factor used in conversion from
c                             integer input values to real numbers
c      limit          (r)  - positive value that sets a limit for
c                             "wet" values; if the absolute value of
c                             an input value is less than 'limit', it
c                             is considered "wet"

c     Above:
c      i  - integer variable [ 4 byte, aka. "long" ]
c      iA - integer array    [ 4 byte, aka. "long" ]
c      i2A- integer*2 array  [ 2 byte, aka. "short" ]
c      r  - real variable    [ 4 byte, aka. "float" ]
c      r8A- real*8 array     [ 8 byte, aka. "double" ]


      subroutine read_set(nx,ny,i2_field,i4_field,r4_field,r8_field,
     >                    n_data,sum,nc_id,var_id,var_type,
     >                    start,count,offset,scale,limit)

      implicit none

      include 'netcdf.inc'

c     Input:
      integer   nx,ny
      integer   nc_id,var_id,var_type,start(4),count(4)
      real      offset,scale,limit

c     Output:
      integer   n_data(nx,ny)
      real*8    sum(nx,ny)

c     Work:
      integer*2 i2_field(nx,ny)
      integer   i4_field(nx,ny)
      real      r4_field(nx,ny)
      real*8    r8_field(nx,ny)

c     Local:
      integer x,y,nc_status


c     Read original field:

      if (var_type .eq. NF_SHORT) then

         nc_status = NF_GET_VARA_INT2(nc_id, var_id, start, count,
     >                                i2_field)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status,331)
c        Convert to real numbers:
         call short2float_2D(nx,ny,i2_field,offset,scale,r4_field)

      else if (var_type .eq. NF_INT) then

         nc_status = NF_GET_VARA_INT(nc_id, var_id, start, count,
     >                               i4_field)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status,332)
c        Convert to real numbers:
         call int2float_2D(nx,ny,i4_field,offset,scale,r4_field)

      else if (var_type .eq. NF_FLOAT) then

         nc_status = NF_GET_VARA_REAL(nc_id, var_id, start, count,
     >                                r4_field)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status,333)

      else if (var_type .eq. NF_DOUBLE) then

         nc_status = NF_GET_VARA_DOUBLE(nc_id, var_id, start, count,
     >                                  r8_field)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status,334)
         write(*,*)'NOTE! Accuracy is lost from real*8 input.'
         do y = 1,ny
            do x = 1,nx
               r4_field(x,y) = sngl(r8_field(x,y))
            end do
         end do
      else
         go to 999
      end if

c     Update 'n_data', 'sum':
      do y = 1,ny
         do x = 1,nx
            if (abs(r4_field(x,y)) .lt. limit) then
               n_data(x,y) = n_data(x,y) + 1
               sum(x,y)    = sum(x,y)    + dble(r4_field(x,y))
            end if
         end do
      end do


      go to 1000

 999  write(*,*)
      write(*,*)'Error in the subroutine "read_set".'
      write(*,*)' Unrecognized type of variable:',var_type
      write(*,*)' Recognized:'
      write(*,*)'  NF_SHORT:  ',NF_SHORT
      write(*,*)'  NF_INT:    ',NF_INT
      write(*,*)'  NF_FLOAT:  ',NF_FLOAT
      write(*,*)'  NF_DOUBLE: ',NF_DOUBLE
      write(*,*)'Terminates.'
      write(*,*)
      nc_status = NF_CLOSE(nc_id)
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status,894)
      stop

 1000 return
      end



c     ********************
c     General description:
c     ********************

c     The subroutine "copy_lonlat" copies 2D or 1D longitudes and
c     latitudes from one netCDF file ('nc_id') to another
c     ('month_id'). The choice of 2D vs. 1D is determined by the
c     value of 'sphere_on' on the include file "nc_dims.inc".


c     ***********************
c     Input/output variables:
c     ***********************

c      month_id       (i)  - the file ID for the output netCDF file
c      nc_id          (i)  - the file ID for the input netCDF file

c     Above:
c      i  - integer variable [ 4 byte, aka. "long" ]


      subroutine copy_lonlat(month_id,nc_id)

      implicit none

      include 'netcdf.inc'
      include 'nc_dims.inc'

c     I/O:
      integer month_id,nc_id

c     Local:
      integer nc_status,lon_out_id,lat_out_id,lon_id,lat_id
      real lon_1d(nx),lat_1d(ny),lon_2d(nx,ny),lat_2d(nx,ny)


      nc_status = NF_INQ_VARID(month_id, 'lon', lon_out_id)
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status,341)
      nc_status = NF_INQ_VARID(month_id, 'lat', lat_out_id)
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status,342)

      nc_status = NF_INQ_VARID(nc_id, 'lon', lon_id)
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status,343)
      nc_status = NF_INQ_VARID(nc_id, 'lat', lat_id)
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status,344)

      if (sphere_on .eq. 1) then

         nc_status = NF_GET_VAR_REAL(nc_id, lon_id, lon_1d)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status,351)
         nc_status = NF_PUT_VAR_REAL(month_id, lon_out_id, lon_1d)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status,352)

         nc_status = NF_GET_VAR_REAL(nc_id, lat_id, lat_1d)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status,353)
         nc_status = NF_PUT_VAR_REAL(month_id, lat_out_id, lat_1d)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status,354)

      else

         nc_status = NF_GET_VAR_REAL(nc_id, lon_id, lon_2d)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status,356)
         nc_status = NF_PUT_VAR_REAL(month_id, lon_out_id, lon_2d)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status,357)
         nc_status = NF_GET_VAR_REAL(nc_id, lat_id, lat_2d)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status,358)
         nc_status = NF_PUT_VAR_REAL(month_id, lat_out_id, lat_2d)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status,359)

      end if

      return
      end



c     ********************
c     General description:
c     ********************

c     The subroutine "get_var_type" reads the input variable type,
c     prints a warning if this is not the same as the input type of the
c     first input field. If the input field is of some integer type,
c     the offset and scaling factor is read from the input file. If
c     the input indicates that this is the first field, and the output
c     field is of an integer type, the input offset and scaling factor
c     is copied to the output file.


c     ***********************
c     Input/output variables:
c     ***********************

c      nc_id          (i)  - the file ID for the input netCDF file
c      month_id       (i)  - the file ID for the output netCDF file
c      var_id         (i)  - the variable ID for the input field
c      var_out_id     (i)  - the variable ID for the output field
c      file_no        (i)  - the input file no. to consider
c      z              (i)  - present level no. (>=1), if value is 1,
c                             write month no. on the netCDF file
c      org_type       (i)  - the variable type of the first field
c      var_type       (i)  - the variable type of the next field,
c                             read below
c      var_type_out   (i)  - the variable type used for the output
c                             field (NF_SHORT, NF_INT, NF_FLOAT)
c      offset_out     (r)  - offset value used in conversion from
c                             floating point to integer numbers
c      scale_out      (r)  - scaling factor used in conversion from
c                             floating point to integer numbers
c      offset         (r)  - offset value used in conversion from
c                             integer input values to real numbers
c      scale          (r)  - scaling factor used in conversion from
c                             integer input values to real numbers

c     Above:
c      i  - integer variable [ 4 byte, aka. "long" ]
c      r  - real variable    [ 4 byte, aka. "float" ]


      subroutine get_vartype(nc_id,month_id,var_id,var_out_id,file_no,z,
     >                       org_type,var_type,var_type_out,
     >                       offset_out,scale_out,offset,scale)

      implicit none

      include 'netcdf.inc'
      include 'nc_dims.inc'

c     Input:
      integer file_no,z,nc_id,var_id,month_id

c     I/O:
      integer var_out_id,org_type,var_type_out
      real offset_out,scale_out,offset,scale

c     Output:
      integer var_type

c     Local:
      integer nc_status


c     Read the input variable type:
      nc_status = NF_INQ_VARTYPE(nc_id, var_id, var_type)
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status,361)

      if (file_no .eq. 1 .and. z .eq. 1) then

c        Store:
         org_type = var_type

c        Find the output variable's type:
         nc_status = NF_INQ_VARTYPE(month_id, var_out_id, var_type_out)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status,362)

c        Checks for consistent initial type:
         if (var_type     .ne. var_type_out .and.
     >       var_type_out .ne. NF_FLOAT) go to 999

c        If type 'NF_SHORT' or 'NF_INT', copy 'offset', 'scale':
         if ((var_type .eq. NF_SHORT .or. var_type .eq. NF_INT) .and.
     >       var_type_out .ne. NF_FLOAT) then
            nc_status = NF_COPY_ATT(nc_id, var_id, 'add_offset',
     >                              month_id, var_out_id)
            if (nc_status .ne. NF_NOERR) call handle_err(nc_status,363)
            nc_status = NF_COPY_ATT(nc_id, var_id, 'scale_factor',
     >                              month_id, var_out_id)
            if (nc_status .ne. NF_NOERR) call handle_err(nc_status,364)
         end if

      else

c        Check:
         if (org_type .ne. var_type) then
            write(*,*)'WARNING!'
            write(*,*)' New variable type encountered for file,',
     >                ' level:',file_no,min(z,nz)
            write(*,*)' ... original, new type:',org_type,var_type
            write(*,*)' continuing ...'
         end if

      end if

c     If type 'NF_SHORT' or 'NF_INT', set 'offset', 'scale':
      if (var_type .eq. NF_SHORT .or. var_type .eq. NF_INT) then
         nc_status = NF_GET_ATT_REAL(nc_id, var_id, 'add_offset',
     >                               offset)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status,365)
         nc_status = NF_GET_ATT_REAL(nc_id, var_id,
     >                               'scale_factor', scale)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status,366)
c        Store?
         if (file_no .eq. 1 .and. z .eq. 1) then
            offset_out = offset
            scale_out  = scale
         end if
      end if


      go to 1000

 999  write(*,*)
      write(*,*)'Error in the subroutine "get_vartype".'
      write(*,*)' Non-matching types of variables:'
      write(*,*)'  input:  ',var_type
      write(*,*)'  output: ',var_type_out
      write(*,*)'Terminates.'
      write(*,*)
      nc_status = NF_CLOSE(nc_id)
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status,895)
      nc_status = NF_CLOSE(month_id)
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status,896)
      stop

 1000 return
      end
